#include <stdio.h>
#include <stdlib.h>

#include "bflib_fileio.h"
#include "globals.h"

#define FEXT_LEN 16
#define FNAME_LEN 255

enum MapFileMode {
  MFM_EndList    = 0,
  MFM_Irrelevant = 1,
  MFM_Optional   = 2,
  MFM_Required   = 3,
  MFM_Crucial    = 4,
  };

typedef long (*FileBodyModFunc)(unsigned char *buf,unsigned long len);

long modify_lif_data(unsigned char *buf,unsigned long len);


struct MapFile {
  short mode;
  char fext[FEXT_LEN];
  FileBodyModFunc mod_func;
  };

const struct MapFile map_files[] = {
  { MFM_Crucial,   "SLB", NULL },
  { MFM_Crucial,   "TNG", NULL },
  { MFM_Crucial,   "OWN", NULL },
  { MFM_Crucial,   "TXT", NULL },
  { MFM_Required,  "APT", NULL },
  { MFM_Required,  "CLM", NULL },
  { MFM_Required,  "DAT", NULL },
  { MFM_Required,  "WIB", NULL },
  { MFM_Required,  "INF", NULL },
  { MFM_Optional,  "LGT", NULL },
  { MFM_Optional,  "WLB", NULL },
  { MFM_Optional,  "VSN", NULL },
  { MFM_Optional,  "ADI", NULL },
  { MFM_Optional,  "LIF", NULL },
  { MFM_Irrelevant,"FLG", NULL },
  { MFM_Irrelevant,"EFF", NULL },
  { MFM_Irrelevant,"OLD", NULL },
  { MFM_Irrelevant,"ORI", NULL },
  { MFM_Irrelevant,"DOC", NULL },
  { MFM_Irrelevant,"BAK", NULL },
  { MFM_Irrelevant,"TMN", NULL },
  { MFM_Irrelevant,"NFO", NULL },
  { MFM_Irrelevant,"BMP", NULL },
  { MFM_EndList,   "",    NULL },
  };

long modify_lif_data(unsigned char *buf,unsigned long len)
{
//TODO
}

short dk1_map_rename_all_files(const char *fn_old, const char *fn_new,short char_case)
{
  char fname_old[FNAME_LEN];
  char fname_new[FNAME_LEN];
  short result;
  int i;
  i = 0;
  result = MFM_EndList;
  while (map_files[i].mode != MFM_EndList)
  {
    sprintf(fname_old,"%s.%s",fn_old,map_files[i].fext);
    sprintf(fname_new,"%s.%s",fn_new,map_files[i].fext);
    switch (char_case)
    {
    case 1:
      strlwr(fname_new);
      break;
    case 2:
      strupr(fname_new);
      break;
    }
    if (LbFileRename(fname_old, fname_new) == -1)
    {
      if (result < map_files[i].mode)
        result = map_files[i].mode;
    } else
    if (map_files[i].mod_func != NULL)
    {
    // TODO
    }
    i++;
  }
  return result;
}

short dk1_map_rename_all_files_num(const unsigned long num_old, const unsigned long num_new, short char_case, short show_msg)
{
  short result;
  char fn_old[FNAME_LEN];
  char fn_new[FNAME_LEN];
  sprintf(fn_old,"MAP%05lu",num_old);
  sprintf(fn_new,"MAP%05lu",num_new);
  if (show_msg)
    printf("Renaming \"%s\" into \"%s\"...",fn_old,fn_new);
  result = dk1_map_rename_all_files(fn_old, fn_new, char_case);
  if (show_msg)
  {
    switch (result)
    {
    case MFM_EndList:
        printf("ALL possible files renamed.\n");
        break;
    case MFM_Irrelevant:
        printf("Some irrelevant files not renamed.\n");
        break;
    case MFM_Optional:
        printf("Some optional files not renamed.\n");
        break;
    case MFM_Required:
        printf("Some REQUIRED files not renamed!\n");
        break;
    case MFM_Crucial:
        printf("Some CRUCIAL files not renamed!!!\n");
        break;
    default:
        printf("Unknown result!?\n");
        break;
    }
  }
  return result;
}

void show_usage(void)
{
    printf("Usage:\n");
    printf("  %s <old_num> <new_num> [count]\n",PROGRAM_NAME);
    printf("Where:\n");
    printf("  <old_num> is source map number, 1..99999\n");
    printf("  <new_num> is destination map number, 1..99999\n");
    printf("  [count]   is amount of maps to rename (default=1)\n");
    printf("\n");
    system("PAUSE");	
}

int main(int argc, char *argv[])
{
  long count;
  long num_old;
  long num_new;
  short result;
  long i;
  printf("\nDungeon Keeper Map Files Renamer");
  printf("\n-------------------------------\n");
  if (argc<3)
  {
    printf("Not enought parameters.\n");
    show_usage();
    return 1;
  }
  num_old = atoi(argv[1]);
  num_new = atoi(argv[2]);
  if (argc>=4)
  {
    count = atoi(argv[3]);
  } else
  {
    count = 1;
  }
  if ((num_old<1) || (num_new<1) || (count<1))
  {
    printf("Invalid command line parameters.\n");
    show_usage();
    return 1;
  }
  for (i=0; i<count; i++)
  {
    result = dk1_map_rename_all_files_num(num_old+i, num_new+i, 1, 1);
    if (result == MFM_Crucial)
    {
      printf("Error when renaming map no %ul. Execution stopped.\n",num_old+i);
      system("PAUSE");	
      return 1;
    }
  }
  printf("\nAll %l maps renamed. Tool author: Mefistotelis\n",count);
  return 0;
}
